# orq-ai-sdk

Developer-friendly & type-safe Python SDK specifically catered to leverage *orq-ai-sdk* API.

<div align="left">
    <a href="https://www.speakeasy.com/?utm_source=orq-ai-sdk&utm_campaign=python"><img src="https://custom-icon-badges.demolab.com/badge/-Built%20By%20Speakeasy-212015?style=for-the-badge&logoColor=FBE331&logo=speakeasy&labelColor=545454" /></a>
    <a href="https://opensource.org/licenses/MIT">
        <img src="https://img.shields.io/badge/License-MIT-blue.svg" style="width: 100px; height: 28px;" />
    </a>
</div>

<!-- Start Summary [summary] -->
## Summary

orq.ai API: orq.ai API documentation

For more information about the API: [orq.ai Documentation](https://docs.orq.ai)
<!-- End Summary [summary] -->

<!-- Start Table of Contents [toc] -->
## Table of Contents
<!-- $toc-max-depth=2 -->
* [orq-ai-sdk](https://github.com/orq-ai/orq-python/blob/master/#orq-ai-sdk)
  * [SDK Installation](https://github.com/orq-ai/orq-python/blob/master/#sdk-installation)
  * [IDE Support](https://github.com/orq-ai/orq-python/blob/master/#ide-support)
  * [SDK Example Usage](https://github.com/orq-ai/orq-python/blob/master/#sdk-example-usage)
  * [Authentication](https://github.com/orq-ai/orq-python/blob/master/#authentication)
  * [Available Resources and Operations](https://github.com/orq-ai/orq-python/blob/master/#available-resources-and-operations)
  * [Server-sent event streaming](https://github.com/orq-ai/orq-python/blob/master/#server-sent-event-streaming)
  * [File uploads](https://github.com/orq-ai/orq-python/blob/master/#file-uploads)
  * [Retries](https://github.com/orq-ai/orq-python/blob/master/#retries)
  * [Error Handling](https://github.com/orq-ai/orq-python/blob/master/#error-handling)
  * [Server Selection](https://github.com/orq-ai/orq-python/blob/master/#server-selection)
  * [Custom HTTP Client](https://github.com/orq-ai/orq-python/blob/master/#custom-http-client)
  * [Resource Management](https://github.com/orq-ai/orq-python/blob/master/#resource-management)
  * [Debugging](https://github.com/orq-ai/orq-python/blob/master/#debugging)
* [Development](https://github.com/orq-ai/orq-python/blob/master/#development)
  * [Maturity](https://github.com/orq-ai/orq-python/blob/master/#maturity)
  * [Contributions](https://github.com/orq-ai/orq-python/blob/master/#contributions)

<!-- End Table of Contents [toc] -->

<!-- Start SDK Installation [installation] -->
## SDK Installation

> [!NOTE]
> **Python version upgrade policy**
>
> Once a Python version reaches its [official end of life date](https://devguide.python.org/versions/), a 3-month grace period is provided for users to upgrade. Following this grace period, the minimum python version supported in the SDK will be updated.

The SDK can be installed with either *pip* or *poetry* package managers.

### PIP

*PIP* is the default package installer for Python, enabling easy installation and management of packages from PyPI via the command line.

```bash
pip install orq-ai-sdk
```

### Poetry

*Poetry* is a modern tool that simplifies dependency management and package publishing by using a single `pyproject.toml` file to handle project metadata and dependencies.

```bash
poetry add orq-ai-sdk
```

### Shell and script usage with `uv`

You can use this SDK in a Python shell with [uv](https://docs.astral.sh/uv/) and the `uvx` command that comes with it like so:

```shell
uvx --from orq-ai-sdk python
```

It's also possible to write a standalone Python script without needing to set up a whole project like so:

```python
#!/usr/bin/env -S uv run --script
# /// script
# requires-python = ">=3.9"
# dependencies = [
#     "orq-ai-sdk",
# ]
# ///

from orq_ai_sdk import Orq

sdk = Orq(
  # SDK arguments
)

# Rest of script here...
```

Once that is saved to a file, you can run it with `uv run script.py` where
`script.py` can be replaced with the actual file name.
<!-- End SDK Installation [installation] -->

<!-- Start IDE Support [idesupport] -->
## IDE Support

### PyCharm

Generally, the SDK will work well with most IDEs out of the box. However, when using PyCharm, you can enjoy much better integration with Pydantic by installing an additional plugin.

- [PyCharm Pydantic Plugin](https://docs.pydantic.dev/latest/integrations/pycharm/)
<!-- End IDE Support [idesupport] -->

<!-- Start SDK Example Usage [usage] -->
## SDK Example Usage

### Example

```python
# Synchronous Example
from orq_ai_sdk import Orq
import os


with Orq(
    api_key=os.getenv("ORQ_API_KEY", ""),
) as orq:

    res = orq.contacts.create(request={
        "external_id": "user_12345",
        "display_name": "Jane Smith",
        "email": "jane.smith@example.com",
        "avatar_url": "https://example.com/avatars/jane-smith.jpg",
        "tags": [
            "premium",
            "beta-user",
            "enterprise",
        ],
        "metadata": {
            "department": "Engineering",
            "role": "Senior Developer",
            "subscription_tier": "premium",
            "last_login": "2024-01-15T10:30:00Z",
        },
    })

    assert res is not None

    # Handle response
    print(res)
```

</br>

The same SDK client can also be used to make asychronous requests by importing asyncio.
```python
# Asynchronous Example
import asyncio
from orq_ai_sdk import Orq
import os

async def main():

    async with Orq(
        api_key=os.getenv("ORQ_API_KEY", ""),
    ) as orq:

        res = await orq.contacts.create_async(request={
            "external_id": "user_12345",
            "display_name": "Jane Smith",
            "email": "jane.smith@example.com",
            "avatar_url": "https://example.com/avatars/jane-smith.jpg",
            "tags": [
                "premium",
                "beta-user",
                "enterprise",
            ],
            "metadata": {
                "department": "Engineering",
                "role": "Senior Developer",
                "subscription_tier": "premium",
                "last_login": "2024-01-15T10:30:00Z",
            },
        })

        assert res is not None

        # Handle response
        print(res)

asyncio.run(main())
```
<!-- End SDK Example Usage [usage] -->

<!-- Start Authentication [security] -->
## Authentication

### Per-Client Security Schemes

This SDK supports the following security scheme globally:

| Name      | Type | Scheme      | Environment Variable |
| --------- | ---- | ----------- | -------------------- |
| `api_key` | http | HTTP Bearer | `ORQ_API_KEY`        |

To authenticate with the API the `api_key` parameter must be set when initializing the SDK client instance. For example:
```python
from orq_ai_sdk import Orq
import os


with Orq(
    api_key=os.getenv("ORQ_API_KEY", ""),
) as orq:

    res = orq.contacts.create(request={
        "external_id": "user_12345",
        "display_name": "Jane Smith",
        "email": "jane.smith@example.com",
        "avatar_url": "https://example.com/avatars/jane-smith.jpg",
        "tags": [
            "premium",
            "beta-user",
            "enterprise",
        ],
        "metadata": {
            "department": "Engineering",
            "role": "Senior Developer",
            "subscription_tier": "premium",
            "last_login": "2024-01-15T10:30:00Z",
        },
    })

    assert res is not None

    # Handle response
    print(res)

```
<!-- End Authentication [security] -->

<!-- Start Available Resources and Operations [operations] -->
## Available Resources and Operations

<details open>
<summary>Available methods</summary>

### [contacts](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/contacts/README.md)

* [create](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/contacts/README.md#create) - Create a contact
* [list](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/contacts/README.md#list) - List contacts
* [retrieve](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/contacts/README.md#retrieve) - Retrieve a contact
* [update](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/contacts/README.md#update) - Update a contact
* [delete](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/contacts/README.md#delete) - Delete a contact

### [datasets](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md)

* [list](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#list) - List datasets
* [create](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#create) - Create a dataset
* [retrieve](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#retrieve) - Retrieve a dataset
* [update](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#update) - Update a dataset
* [delete](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#delete) - Delete a dataset
* [list_datapoints](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#list_datapoints) - List datapoints
* [create_datapoint](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#create_datapoint) - Create a datapoint
* [retrieve_datapoint](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#retrieve_datapoint) - Retrieve a datapoint
* [update_datapoint](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#update_datapoint) - Update a datapoint
* [delete_datapoint](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#delete_datapoint) - Delete a datapoint
* [clear](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/datasets/README.md#clear) - Delete all datapoints

### [deployments](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/deploymentssdk/README.md)

* [list](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/deploymentssdk/README.md#list) - List all deployments
* [get_config](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/deploymentssdk/README.md#get_config) - Get config
* [invoke](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/deploymentssdk/README.md#invoke) - Invoke
* [stream](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/deploymentssdk/README.md#stream) - Stream

#### [deployments.metrics](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/metrics/README.md)

* [create](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/metrics/README.md#create) - Add metrics

### [evals](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md)

* [all](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#all) - Get all Evaluators
* [create](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#create) - Create an Evaluator
* [update](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#update) - Update an Evaluator
* [delete](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#delete) - Delete an Evaluator
* [bert_score](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#bert_score) - Run BertScore Evaluator
* [bleu_score](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#bleu_score) - Run BLEU Score Evaluator
* [contains_all](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#contains_all) - Run Contains All Evaluator
* [contains_any](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#contains_any) - Run Contains Any Evaluator
* [contains_email](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#contains_email) - Run Contains Email Evaluator
* [contains_none](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#contains_none) - Run Contains None Evaluator
* [contains_url](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#contains_url) - Run Contains URL Evaluator
* [contains_valid_link](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#contains_valid_link) - Run Contains Valid Link Evaluator
* [contains](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#contains) - Run Contains Evaluator
* [ends_with](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ends_with) - Run Ends With Evaluator
* [exact_match](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#exact_match) - Run Exact Match Evaluator
* [length_between](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#length_between) - Run Length Between Evaluator
* [length_greater_than](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#length_greater_than) - Run Length Greater Than Evaluator
* [length_less_than](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#length_less_than) - Run Length Less Than Evaluator
* [valid_json](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#valid_json) - Run JSON Validation Evaluator
* [age_appropriate](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#age_appropriate) - Run Age Appropriate Evaluator
* [bot_detection](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#bot_detection) - Run Bot Detection Evaluator
* [fact_checking_knowledge_base](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#fact_checking_knowledge_base) - Run Fact Checking Knowledge Base Evaluator
* [grammar](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#grammar) - Run Grammar Evaluator
* [localization](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#localization) - Run Localization Evaluator
* [pii](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#pii) - Run PII Evaluator
* [sentiment_classification](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#sentiment_classification) - Run Sentiment Classification Evaluator
* [summarization](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#summarization) - Run Summarization Evaluator
* [tone_of_voice](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#tone_of_voice) - Run Tone of Voice Evaluator
* [translation](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#translation) - Run Translation Evaluator
* [ragas_coherence](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_coherence) - Run Coherence Evaluator
* [ragas_conciseness](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_conciseness) - Run Conciseness Evaluator
* [ragas_context_precision](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_context_precision) - Run Context Precision Evaluator
* [ragas_context_recall](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_context_recall) - Run Context Recall Evaluator
* [ragas_context_entities_recall](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_context_entities_recall) - Run Context Entities Recall Evaluator
* [ragas_correctness](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_correctness) - Run Correctness Evaluator
* [ragas_faithfulness](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_faithfulness) - Run Faithfulness Evaluator
* [ragas_harmfulness](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_harmfulness) - Run Harmfulness Evaluator
* [ragas_maliciousness](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_maliciousness) - Run Maliciousness Evaluator
* [ragas_noise_sensitivity](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_noise_sensitivity) - Run Noise Sensitivity Evaluator
* [ragas_response_relevancy](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_response_relevancy) - Run Response Relevancy Evaluator
* [ragas_summarization](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#ragas_summarization) - Run Summarization Evaluator
* [invoke](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/evals/README.md#invoke) - Invoke a Custom Evaluator

### [feedback](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/feedback/README.md)

* [create](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/feedback/README.md#create) - Submit feedback

### [files](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/files/README.md)

* [create](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/files/README.md#create) - Create file
* [list](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/files/README.md#list) - List all files
* [get](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/files/README.md#get) - Retrieve a file
* [delete](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/files/README.md#delete) - Delete file

### [knowledge](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md)

* [list](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#list) - List all knowledge bases
* [retrieve](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#retrieve) - Retrieves a knowledge base
* [update](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#update) - Updates a knowledge
* [delete](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#delete) - Deletes a knowledge
* [search](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#search) - Search knowledge base
* [list_datasources](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#list_datasources) - List all datasources
* [create_datasource](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#create_datasource) - Create a new datasource
* [retrieve_datasource](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#retrieve_datasource) - Retrieve a datasource
* [delete_datasource](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#delete_datasource) - Deletes a datasource
* [update_datasource](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#update_datasource) - Update a datasource
* [create_chunks](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#create_chunks) - Create chunks for a datasource
* [list_chunks](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#list_chunks) - List all chunks for a datasource
* [update_chunk](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#update_chunk) - Update a chunk
* [delete_chunk](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#delete_chunk) - Delete a chunk
* [retrieve_chunk](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#retrieve_chunk) - Retrieve a chunk
* [chunk_text](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/knowledge/README.md#chunk_text) - Chunk text content using various strategies

### [models](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/models/README.md)

* [list](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/models/README.md#list) - List models


### [prompts](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/prompts/README.md)

* [list](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/prompts/README.md#list) - List all prompts
* [create](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/prompts/README.md#create) - Create a prompt
* [retrieve](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/prompts/README.md#retrieve) - Retrieve a prompt
* [update](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/prompts/README.md#update) - Update a prompt
* [delete](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/prompts/README.md#delete) - Delete a prompt
* [list_versions](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/prompts/README.md#list_versions) - List all prompt versions
* [get_version](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/prompts/README.md#get_version) - Retrieve a prompt version

### [remoteconfigs](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/remoteconfigs/README.md)

* [retrieve](https://github.com/orq-ai/orq-python/blob/master/docs/sdks/remoteconfigs/README.md#retrieve) - Retrieve a remote config

</details>
<!-- End Available Resources and Operations [operations] -->

<!-- Start Server-sent event streaming [eventstream] -->
## Server-sent event streaming

[Server-sent events][mdn-sse] are used to stream content from certain
operations. These operations will expose the stream as [Generator][generator] that
can be consumed using a simple `for` loop. The loop will
terminate when the server no longer has any events to send and closes the
underlying connection.  

The stream is also a [Context Manager][context-manager] and can be used with the `with` statement and will close the
underlying connection when the context is exited.

```python
from orq_ai_sdk import Orq
import os


with Orq(
    environment="<value>",
    contact_id="<id>",
    api_key=os.getenv("ORQ_API_KEY", ""),
) as orq:

    res = orq.deployments.stream(key="<key>")

    assert res is not None

    with res as event_stream:
        for event in event_stream:
            # handle event
            print(event, flush=True)

```

[mdn-sse]: https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events
[generator]: https://book.pythontips.com/en/latest/generators.html
[context-manager]: https://book.pythontips.com/en/latest/context_managers.html
<!-- End Server-sent event streaming [eventstream] -->

<!-- Start File uploads [file-upload] -->
## File uploads

Certain SDK methods accept file objects as part of a request body or multi-part request. It is possible and typically recommended to upload files as a stream rather than reading the entire contents into memory. This avoids excessive memory consumption and potentially crashing with out-of-memory errors when working with very large files. The following example demonstrates how to attach a file stream to a request.

> [!TIP]
>
> For endpoints that handle file uploads bytes arrays can also be used. However, using streams is recommended for large files.
>

```python
from orq_ai_sdk import Orq
import os


with Orq(
    api_key=os.getenv("ORQ_API_KEY", ""),
) as orq:

    res = orq.files.create(file={
        "file_name": "example.file",
        "content": open("example.file", "rb"),
    }, purpose="retrieval")

    assert res is not None

    # Handle response
    print(res)

```
<!-- End File uploads [file-upload] -->

<!-- Start Retries [retries] -->
## Retries

Some of the endpoints in this SDK support retries. If you use the SDK without any configuration, it will fall back to the default retry strategy provided by the API. However, the default retry strategy can be overridden on a per-operation basis, or across the entire SDK.

To change the default retry strategy for a single API call, simply provide a `RetryConfig` object to the call:
```python
from orq_ai_sdk import Orq
from orq_ai_sdk.utils import BackoffStrategy, RetryConfig
import os


with Orq(
    api_key=os.getenv("ORQ_API_KEY", ""),
) as orq:

    res = orq.contacts.create(request={
        "external_id": "user_12345",
        "display_name": "Jane Smith",
        "email": "jane.smith@example.com",
        "avatar_url": "https://example.com/avatars/jane-smith.jpg",
        "tags": [
            "premium",
            "beta-user",
            "enterprise",
        ],
        "metadata": {
            "department": "Engineering",
            "role": "Senior Developer",
            "subscription_tier": "premium",
            "last_login": "2024-01-15T10:30:00Z",
        },
    },
        RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False))

    assert res is not None

    # Handle response
    print(res)

```

If you'd like to override the default retry strategy for all operations that support retries, you can use the `retry_config` optional parameter when initializing the SDK:
```python
from orq_ai_sdk import Orq
from orq_ai_sdk.utils import BackoffStrategy, RetryConfig
import os


with Orq(
    retry_config=RetryConfig("backoff", BackoffStrategy(1, 50, 1.1, 100), False),
    api_key=os.getenv("ORQ_API_KEY", ""),
) as orq:

    res = orq.contacts.create(request={
        "external_id": "user_12345",
        "display_name": "Jane Smith",
        "email": "jane.smith@example.com",
        "avatar_url": "https://example.com/avatars/jane-smith.jpg",
        "tags": [
            "premium",
            "beta-user",
            "enterprise",
        ],
        "metadata": {
            "department": "Engineering",
            "role": "Senior Developer",
            "subscription_tier": "premium",
            "last_login": "2024-01-15T10:30:00Z",
        },
    })

    assert res is not None

    # Handle response
    print(res)

```
<!-- End Retries [retries] -->

<!-- Start Error Handling [errors] -->
## Error Handling

[`OrqError`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/orqerror.py) is the base class for all HTTP error responses. It has the following properties:

| Property           | Type             | Description                                                                             |
| ------------------ | ---------------- | --------------------------------------------------------------------------------------- |
| `err.message`      | `str`            | Error message                                                                           |
| `err.status_code`  | `int`            | HTTP response status code eg `404`                                                      |
| `err.headers`      | `httpx.Headers`  | HTTP response headers                                                                   |
| `err.body`         | `str`            | HTTP body. Can be empty string if no body is returned.                                  |
| `err.raw_response` | `httpx.Response` | Raw HTTP response                                                                       |
| `err.data`         |                  | Optional. Some errors may contain structured data. [See Error Classes](https://github.com/orq-ai/orq-python/blob/master/#error-classes). |

### Example
```python
from orq_ai_sdk import Orq, models
import os


with Orq(
    api_key=os.getenv("ORQ_API_KEY", ""),
) as orq:
    res = None
    try:

        res = orq.contacts.retrieve(id="<id>")

        assert res is not None

        # Handle response
        print(res)


    except models.OrqError as e:
        # The base class for HTTP error responses
        print(e.message)
        print(e.status_code)
        print(e.body)
        print(e.headers)
        print(e.raw_response)

        # Depending on the method different errors may be thrown
        if isinstance(e, models.RetrieveContactContactsResponseBody):
            print(e.data.error)  # str
```

### Error Classes
**Primary error:**
* [`OrqError`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/orqerror.py): The base class for HTTP error responses.

<details><summary>Less common errors (91)</summary>

<br />

**Network errors:**
* [`httpx.RequestError`](https://www.python-httpx.org/exceptions/#httpx.RequestError): Base class for request errors.
    * [`httpx.ConnectError`](https://www.python-httpx.org/exceptions/#httpx.ConnectError): HTTP client was unable to make a request to a server.
    * [`httpx.TimeoutException`](https://www.python-httpx.org/exceptions/#httpx.TimeoutException): HTTP request timed out.


**Inherit from [`OrqError`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/orqerror.py)**:
* [`HonoAPIError`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/honoapierror.py): Applicable to 6 of 93 methods.*
* [`RetrieveContactContactsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/retrievecontactcontactsresponsebody.py): Contact not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`UpdateContactContactsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/updatecontactcontactsresponsebody.py): Contact not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`DeleteContactResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/deletecontactresponsebody.py): Contact not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`UpdatePromptPromptsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/updatepromptpromptsresponsebody.py): Prompt not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`GetPromptVersionPromptsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/getpromptversionpromptsresponsebody.py): Not Found - The prompt or prompt version does not exist. Status code `404`. Applicable to 1 of 93 methods.*
* [`GetEvalsEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/getevalsevalsresponsebody.py): Workspace ID is not found on the request. Status code `404`. Applicable to 1 of 93 methods.*
* [`CreateEvalEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/createevalevalsresponsebody.py): Workspace ID is not found on the request. Status code `404`. Applicable to 1 of 93 methods.*
* [`UpdateEvalEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/updateevalevalsresponsebody.py): Workspace ID is not found on the request. Status code `404`. Applicable to 1 of 93 methods.*
* [`DeleteEvalResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/deleteevalresponsebody.py): Workspace ID is not found on the request. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsBertScoreEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsbertscoreevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsBleuScoreEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsbleuscoreevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsContainsAllEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsallevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsContainsAnyEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsanyevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsContainsEmailEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsemailevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsContainsNoneEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsnoneevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsContainsURLEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsurlevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsContainsValidLinkEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsvalidlinkevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsContainsEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsEndsWithEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsendswithevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsExactMatchEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsexactmatchevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsLengthBetweenEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalslengthbetweenevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsLengthGreaterThanEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalslengthgreaterthanevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsLengthLessThanEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalslengthlessthanevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsValidJSONEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsvalidjsonevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsAgeAppropriateEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsageappropriateevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsBotDetectionEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsbotdetectionevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsFactCheckingKnowledgeBaseEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsfactcheckingknowledgebaseevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsGrammarEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsgrammarevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsLocalizationEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalslocalizationevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsPiiEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalspiievalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsSentimentClassificationEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalssentimentclassificationevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsSummarizationEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalssummarizationevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsToneOfVoiceEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalstoneofvoiceevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsTranslationEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalstranslationevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasCoherenceEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascoherenceevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasConcisenessEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasconcisenessevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasContextPrecisionEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascontextprecisionevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasContextRecallEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascontextrecallevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasContextEntitiesRecallEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascontextentitiesrecallevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasCorrectnessEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascorrectnessevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasFaithfulnessEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasfaithfulnessevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasHarmfulnessEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasharmfulnessevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasMaliciousnessEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasmaliciousnessevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasNoiseSensitivityEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasnoisesensitivityevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasResponseRelevancyEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasresponserelevancyevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsRagasSummarizationEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragassummarizationevalsresponsebody.py): Evaluator not found. Status code `404`. Applicable to 1 of 93 methods.*
* [`InvokeEvalEvalsResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/invokeevalevalsresponsebody.py): Workspace ID is not found on the request. Status code `404`. Applicable to 1 of 93 methods.*
* [`EvalsBertScoreEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsbertscoreevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsBleuScoreEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsbleuscoreevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsContainsAllEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsallevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsContainsAnyEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsanyevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsContainsEmailEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsemailevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsContainsNoneEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsnoneevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsContainsURLEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsurlevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsContainsValidLinkEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsvalidlinkevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsContainsEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalscontainsevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsEndsWithEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsendswithevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsExactMatchEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsexactmatchevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsLengthBetweenEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalslengthbetweenevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsLengthGreaterThanEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalslengthgreaterthanevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsLengthLessThanEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalslengthlessthanevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsValidJSONEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsvalidjsonevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsAgeAppropriateEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsageappropriateevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsBotDetectionEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsbotdetectionevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsFactCheckingKnowledgeBaseEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsfactcheckingknowledgebaseevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsGrammarEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsgrammarevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsLocalizationEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalslocalizationevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsPiiEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalspiievalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsSentimentClassificationEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalssentimentclassificationevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsSummarizationEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalssummarizationevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsToneOfVoiceEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalstoneofvoiceevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsTranslationEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalstranslationevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasCoherenceEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascoherenceevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasConcisenessEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasconcisenessevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasContextPrecisionEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascontextprecisionevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasContextRecallEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascontextrecallevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasContextEntitiesRecallEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascontextentitiesrecallevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasCorrectnessEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragascorrectnessevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasFaithfulnessEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasfaithfulnessevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasHarmfulnessEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasharmfulnessevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasMaliciousnessEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasmaliciousnessevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasNoiseSensitivityEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasnoisesensitivityevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasResponseRelevancyEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragasresponserelevancyevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`EvalsRagasSummarizationEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/evalsragassummarizationevalsresponseresponsebody.py): Internal server error. Status code `500`. Applicable to 1 of 93 methods.*
* [`InvokeEvalEvalsResponseResponseBody`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/invokeevalevalsresponseresponsebody.py): Error running the evaluator. Status code `500`. Applicable to 1 of 93 methods.*
* [`ResponseValidationError`](https://github.com/orq-ai/orq-python/blob/master/./src/orq_ai_sdk/models/responsevalidationerror.py): Type mismatch between the response data and the expected Pydantic model. Provides access to the Pydantic validation error via the `cause` attribute.

</details>

\* Check [the method documentation](https://github.com/orq-ai/orq-python/blob/master/#available-resources-and-operations) to see if the error is applicable.
<!-- End Error Handling [errors] -->

<!-- Start Server Selection [server] -->
## Server Selection

### Override Server URL Per-Client

The default server can be overridden globally by passing a URL to the `server_url: str` optional parameter when initializing the SDK client instance. For example:
```python
from orq_ai_sdk import Orq
import os


with Orq(
    server_url="https://my.orq.ai",
    api_key=os.getenv("ORQ_API_KEY", ""),
) as orq:

    res = orq.contacts.create(request={
        "external_id": "user_12345",
        "display_name": "Jane Smith",
        "email": "jane.smith@example.com",
        "avatar_url": "https://example.com/avatars/jane-smith.jpg",
        "tags": [
            "premium",
            "beta-user",
            "enterprise",
        ],
        "metadata": {
            "department": "Engineering",
            "role": "Senior Developer",
            "subscription_tier": "premium",
            "last_login": "2024-01-15T10:30:00Z",
        },
    })

    assert res is not None

    # Handle response
    print(res)

```
<!-- End Server Selection [server] -->

<!-- Start Custom HTTP Client [http-client] -->
## Custom HTTP Client

The Python SDK makes API calls using the [httpx](https://www.python-httpx.org/) HTTP library.  In order to provide a convenient way to configure timeouts, cookies, proxies, custom headers, and other low-level configuration, you can initialize the SDK client with your own HTTP client instance.
Depending on whether you are using the sync or async version of the SDK, you can pass an instance of `HttpClient` or `AsyncHttpClient` respectively, which are Protocol's ensuring that the client has the necessary methods to make API calls.
This allows you to wrap the client with your own custom logic, such as adding custom headers, logging, or error handling, or you can just pass an instance of `httpx.Client` or `httpx.AsyncClient` directly.

For example, you could specify a header for every request that this sdk makes as follows:
```python
from orq_ai_sdk import Orq
import httpx

http_client = httpx.Client(headers={"x-custom-header": "someValue"})
s = Orq(client=http_client)
```

or you could wrap the client with your own custom logic:
```python
from orq_ai_sdk import Orq
from orq_ai_sdk.httpclient import AsyncHttpClient
import httpx

class CustomClient(AsyncHttpClient):
    client: AsyncHttpClient

    def __init__(self, client: AsyncHttpClient):
        self.client = client

    async def send(
        self,
        request: httpx.Request,
        *,
        stream: bool = False,
        auth: Union[
            httpx._types.AuthTypes, httpx._client.UseClientDefault, None
        ] = httpx.USE_CLIENT_DEFAULT,
        follow_redirects: Union[
            bool, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
    ) -> httpx.Response:
        request.headers["Client-Level-Header"] = "added by client"

        return await self.client.send(
            request, stream=stream, auth=auth, follow_redirects=follow_redirects
        )

    def build_request(
        self,
        method: str,
        url: httpx._types.URLTypes,
        *,
        content: Optional[httpx._types.RequestContent] = None,
        data: Optional[httpx._types.RequestData] = None,
        files: Optional[httpx._types.RequestFiles] = None,
        json: Optional[Any] = None,
        params: Optional[httpx._types.QueryParamTypes] = None,
        headers: Optional[httpx._types.HeaderTypes] = None,
        cookies: Optional[httpx._types.CookieTypes] = None,
        timeout: Union[
            httpx._types.TimeoutTypes, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
        extensions: Optional[httpx._types.RequestExtensions] = None,
    ) -> httpx.Request:
        return self.client.build_request(
            method,
            url,
            content=content,
            data=data,
            files=files,
            json=json,
            params=params,
            headers=headers,
            cookies=cookies,
            timeout=timeout,
            extensions=extensions,
        )

s = Orq(async_client=CustomClient(httpx.AsyncClient()))
```
<!-- End Custom HTTP Client [http-client] -->

<!-- Start Resource Management [resource-management] -->
## Resource Management

The `Orq` class implements the context manager protocol and registers a finalizer function to close the underlying sync and async HTTPX clients it uses under the hood. This will close HTTP connections, release memory and free up other resources held by the SDK. In short-lived Python programs and notebooks that make a few SDK method calls, resource management may not be a concern. However, in longer-lived programs, it is beneficial to create a single SDK instance via a [context manager][context-manager] and reuse it across the application.

[context-manager]: https://docs.python.org/3/reference/datamodel.html#context-managers

```python
from orq_ai_sdk import Orq
import os
def main():

    with Orq(
        api_key=os.getenv("ORQ_API_KEY", ""),
    ) as orq:
        # Rest of application here...


# Or when using async:
async def amain():

    async with Orq(
        api_key=os.getenv("ORQ_API_KEY", ""),
    ) as orq:
        # Rest of application here...
```
<!-- End Resource Management [resource-management] -->

<!-- Start Debugging [debug] -->
## Debugging

You can setup your SDK to emit debug logs for SDK requests and responses.

You can pass your own logger class directly into your SDK.
```python
from orq_ai_sdk import Orq
import logging

logging.basicConfig(level=logging.DEBUG)
s = Orq(debug_logger=logging.getLogger("orq_ai_sdk"))
```

You can also enable a default debug logger by setting an environment variable `ORQ_DEBUG` to true.
<!-- End Debugging [debug] -->

<!-- Placeholder for Future Speakeasy SDK Sections -->

# Development

## Maturity

This SDK is in beta, and there may be breaking changes between versions without a major version update. Therefore, we recommend pinning usage
to a specific package version. This way, you can install the same version each time without breaking changes unless you are intentionally
looking for the latest version.

## Contributions

While we value open-source contributions to this SDK, this library is generated programmatically. Any manual changes added to internal files will be overwritten on the next generation. 
We look forward to hearing your feedback. Feel free to open a PR or an issue with a proof of concept and we'll do our best to include it in a future release. 

### SDK Created by [Speakeasy](https://www.speakeasy.com/?utm_source=orq-ai-sdk&utm_campaign=python)
